'''
Created on Jan 25, 2016

@author: grastogi
'''
import argparse
import json
import logging
import sys
from avi.sdk.avi_api import ApiSession

logger = logging.getLogger(__name__)
ch = logging.StreamHandler(sys.stdout)
root_logger = logging.getLogger()
root_logger.setLevel(logging.DEBUG)
root_logger.addHandler(ch)


def test_multi_tenant(args, tenants):
    '''
    1. get list of tenants
    2. create API for each tenant
    3. call virtual service get on each tenant
    '''
    t_apis = []
    for tenant in tenants:
        api = ApiSession.get_session(args.controller_ip, args.user, args.password,
                     tenant=tenant)
        t_apis.append(api)
        print ' created api for teant %s api %s' % (tenant, api)

    for _ in xrange(5):
        for api in t_apis:
            r = api.get('virtualservice')
            results = json.loads(r.text)['results']
            logger.debug(' tenant %s num vs %d', api.tenant, len(results))

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--controller_ip', help='controller ip')
    parser.add_argument('-u', '--user', default='admin', help='controller ip')
    parser.add_argument('-p', '--password', default='avi123',
                        help='controller ip')

    tenants = []
    args = parser.parse_args()
    logger.debug('parsed args %s', args)

    api = ApiSession.get_session(args.controller_ip, username=args.user,
                 password=args.password, tenant='admin')
    resp = api.get('tenant')
    logger.debug('resp code %d txt %s', resp.status_code, resp.text)

    tenants = (json.loads(resp.text))['results']

    tnames = [tenant['name'] for tenant in tenants]
    test_multi_tenant(args, tnames)



'''
Created on Aug 19, 2016

@author: grastogi
'''
import argparse
import logging
import sys
from requests.packages import urllib3
from avi.sdk.avi_api import ApiSession
from copy import deepcopy

logger = logging.getLogger(__name__)
ch = logging.StreamHandler(sys.stdout)
root_logger = logging.getLogger()
root_logger.setLevel(logging.DEBUG)
root_logger.addHandler(ch)
urllib3.disable_warnings()


class AnalyticsApiExamples(object):
    """
    """
    def __init__(self, api):
        self.api = api

    def fill_params(self, metrics, step, limit, start, stop, params):
        if step:
            params['step'] = step
        if limit:
            params['limit'] = limit
        if start:
            params['start'] = start
        if stop:
            params['stop'] = stop
        if metrics:
            params['metric_id'] = metrics
        return params

    def collections_api(
            self, entity_uuid, pool_uuid, server, metric_ids, step, limit,
            start, stop):
        path = 'analytics/metrics/collection/?'
        params = self.fill_params(metric_ids, step, limit, start, stop)
        creq = {'metric_requests': []}

        # adding first request
        req = deepcopy(params)
        req['entity_uuid'] = entity_uuid
        if pool_uuid:
            req['pool_uuid'] = pool_uuid
        if server:
            req['server'] = server
        





if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', '--option',
                        choices=['collections_api'],
                        help='list of example operations',
                        default='collections_api')

    parser.add_argument('-e', '--entity_uuid',
                        help='VirtualService, SE or VM UUID',
                        default='')
    parser.add_argument(
        '-s', '--server_ips',
        help='Pool Server IPs comma separated Eg. 1.1.1.1,2.2.2.2',
        default='1.1.1.1,1.1.1.2')
    parser.add_argument('-u', '--user', help='controller user',
                        default='admin')
    parser.add_argument('-p', '--password', help='controller user password',
                        default='avi123')
    parser.add_argument('-t', '--tenant', help='tenant name',
                        default=None)
    parser.add_argument('--tenant-uuid', help='tenant uuid',
                        default=None)
    parser.add_argument('-c', '--controller_ip', help='controller ip')
    parser.add_argument('-m', '--metric_id',
                        help='Comma separated metric ids',
                        default='l4_client.avg_bandwidth')
    parser.add_argument('-s', '--step',
                        help='step for analytics APIs',
                        default=300)
    parser.add_argument('-l', '--limit',
                        help='Comma separated metric ids',
                        default=5, type=int)
    parser.add_argument('--start',
                        help='ISO format timestamp',
                        default='')
    parser.add_argument('--stop',
                        help='ISO format timestamp',
                        default='')

    args = parser.parse_args()
    print 'parsed args', args
    api = ApiSession.get_session(
            args.controller_ip, args.user, args.password,
            tenant=args.tenant, tenant_uuid=args.tenant_uuid)

    eg = AnalyticsApiExamples(api)

    fn = getattr(eg, args.option)
    fn(args.entity_uuid, args.pool_uuid, args.server, args.metrics,
       args.step, args.limit, args.start, args.stop)



import re
import os
if __name__ == '__main__':
    version = '0'
    regex = '\(.*?\)'
    dir_path = os.path.abspath(os.path.dirname(__file__))
    with open(dir_path+os.path.sep+'changelog', 'r+') as f:
        content = f.read()
        f.seek(0)
        f.truncate()
        f.write(re.sub(regex,'(%s)' % version, content))



#!/usr/bin/python

import json
import sys

sys.path.insert(1,  '../')

from apic_client import APICClient
from avi.sdk.avi_api import ApiSession
from avi.sdk.utils.api_utils import ApiUtils


tenant_config = None
admin_session = None
apic_session = None

def load_tenant_config():
    global tenant_config
    with open( 'apic_tenant_config.json', 'r' ) as f:
        try:
            tenant_config = json.load(f)
        except Exception, e:
            print e
            sys.exit(0)

def get_avi_controller_config():
    global tenant_config
    return tenant_config['AviController'][0]

def get_avi_apic_config():
    global tenant_config
    if 'Apic' in tenant_config['AviController'][0]:
        return tenant_config['AviController'][0]['Apic'][0]
    return None

def get_avi_pool():
    global tenant_config
    if 'Pool' in tenant_config['AviController'][0]:
        return tenant_config['AviController'][0]['Pool']
    return None

def get_avi_vs():
    global tenant_config
    if 'VirtualService' in tenant_config['AviController'][0]:
        return tenant_config['AviController'][0]['VirtualService']
    return None

def get_avi_ipaddrgroup():
    global tenant_config
    if 'IpAddrGroup' in tenant_config['AviController'][0]:
        return tenant_config['AviController'][0]['IpAddrGroup']
    return None

def get_apic_app_profile():
    global tenant_config
    if 'AppProfile' in tenant_config:
        return tenant_config['AppProfile']
    return None

def get_apic_bd():
    global tenant_config
    if 'BridgeDomain' in tenant_config:
        return tenant_config['BridgeDomain']
    return None

def get_apic_contract():
    global tenant_config
    if 'Contract' in tenant_config:
        return tenant_config['Contract']
    return None

def get_apic_private_network():
    global tenant_config
    if 'PrivateNetwork' in tenant_config:
        return tenant_config['PrivateNetwork']
    return None

def get_apic_graphs():
    global tenant_config
    if 'Graph' in tenant_config:
        return tenant_config['Graph']
    return None


def create_apic_topology():

    print ('------------------------------------------------------')
    print ('Creating objects in APIC for Tenant : %s' %(tenant_config['Tenant']))
    print ('------------------------------------------------------')
    # Setup the Bridge Domains
    apic_bds = get_apic_bd()
    for bd in apic_bds:
        for subnet in bd['subnets']:
            print ('Creating BD : %s Subnet : %s ...'%
                      (bd['name'],subnet['gateway']))
            apic_session.createTenantBD(tenant_config['Tenant'],
                           bd['name'],
                           subnet['gateway'],
                           bd['network'])

    # Setup the Graphs
    apic_graphs = get_apic_graphs()
    apic_config = get_avi_apic_config()
    for graph in apic_graphs:
        print ('Creating Graph : %s ...' %(graph['name']))
        apic_session.createGraph(tenant_config['Tenant'],
                apic_config['vendor'],
                apic_config['product'],
                graph['name'],
                graph['conn_out'],
                graph['conn_int'])

    # Setup the Contracts
    apic_contracts = get_apic_contract()
    for contract in apic_contracts:
        print ('Creating Contract : %s ...' %(contract['name']))
        apic_session.createTenantContract(tenant_config['Tenant'],
                             contract['name'],
                             contract['subject']['graph'],
                             contract['subject']['filter'])

    # Setup the Application Profiles
    apic_apps = get_apic_app_profile()
    for app in apic_apps:
        for epg in app['EpgConfig']:
            print ('Creating Application Profile : %s Epg : %s...' %
                  (app['name'],epg['name']))
            apic_session.createAppEpg(tenant_config['Tenant'],
                 app['name'],
                 epg['name'],
                 epg['vmm_domain'],
                 epg['bridge_domain'])
            for contract in epg['EpgContract']:
                if contract['type'] == 'CONSUMER':
                    apic_session.addEpgContractCons(tenant_config['Tenant'],
                        app['name'],
                        epg['name'],
                        contract['name'])
                if contract['type'] == 'PROVIDER':
                    apic_session.addEpgContractProv(tenant_config['Tenant'],
                        app['name'],
                        epg['name'],
                        contract['name'])

def create_avi_ipaddrgroup(session):
    ipaddrgroup_config = get_avi_ipaddrgroup()
    for group in ipaddrgroup_config:
        print ('Creating IP Address Group : %s...' %(group['name']))
        group_obj = None
        if 'epg' in group:
            group_obj = {
                'name' : group['name'],
                'apic_epg_name' : group['epg']
            }
        else:
            addrs_obj = []
            for ip in group['addrs']:
                addrs_obj.append({
                    'type' : 'V4',
                    'addr' : ip
                })
            group_obj = {
                'name' : group['name'],
                'addrs' : addrs_obj
            }
        session.create_or_update(obj='ipaddrgroup', 
                                 obj_name=group['name'],
                                 data=group_obj)

def create_avi_pool(session):
    pool_config = get_avi_pool()
    for pool in pool_config:
        print ('Creating Pool : %s...' %(pool['name']))
        pool_obj = {
            'name' : pool['name'],
            'apic_epg_name' : pool['pool_epg']
        }
        session.create_or_update(obj='pool', obj_name=pool['name'],
                                 data=pool_obj)

def create_avi_networksecuritypolicy(session, vs):
    network_security = vs['network_security']
    group_refs = []
    for group in network_security['groups']:
        ref = session.get_object_uri(
                  'ipaddrgroup',
                  group)
        group_refs.append(ref)
    ns_policy_name = 'vs-' + vs['name'] + '-ns'
    ns_policy_obj = None
    if network_security['allow'] is True:
        ns_policy_obj = {
            'name' : ns_policy_name,
            'rules' : [
                {
                    'index' : 1,
                    'enable' : True,
                    'name' : 'Allowed IPs',
                    'action' : 'NETWORK_SECURITY_POLICY_ACTION_TYPE_DENY',
                    'match' : {
                        'client_ip' : {
                            'group_refs' : group_refs,
                            'match_criteria' : 'IS_NOT_IN'
                        }
                    },
                    'log' : True
                }
            ]
        }
    else:
        ns_policy_obj = {
            'name' : ns_policy_name,
            'rules' : [
                {
                    'index' : 1,
                    'enable' : True,
                    'name' : 'Blocked IPs',
                    'action' : 'NETWORK_SECURITY_POLICY_ACTION_TYPE_DENY',
                    'match' : {
                        'client_ip' : {
                            'group_refs' : group_refs,
                            'match_criteria' : 'IS_IN'
                        }
                    },
                    'log' : True
                }
            ]
        }
    
    session.create_or_update(obj='networksecuritypolicy',
                             obj_name=ns_policy_name,
                             data=ns_policy_obj)
    return session.get_object_uri(
               'networksecuritypolicy',
               ns_policy_name)

def create_avi_virtualservice(session, app_profile_name):
    vs_config = get_avi_vs()
    app_profile_ref = session.get_object_uri(
                        'applicationprofile', 
                        app_profile_name)
    for vs in vs_config: 
        print ('Creating VirtualService : %s...' %(vs['name']))
        vs_obj = None
        pool_ref = session.get_object_uri('pool', vs['pool'])
        service_objs = []
        for service in vs['services']:
            service_objs.append({
                'port' : service['port'],
                'enable_ssl' : service['enable_ssl']
            })
        network_security_ref = None
        if 'network_security' in vs:
            network_security_ref = create_avi_networksecuritypolicy(
                                       session,
                                       vs)
            vs_obj = {
                'name' : vs['name'],
                'type' : 'VS_TYPE_NORMAL',
                'ip_address' : {
                    'addr' : vs['vip'],
                    'type' : 'V4'
                },
                'enabled' : True,
                'services' : service_objs,
                'pool_ref' : pool_ref,
                'application_profile_ref' : app_profile_ref,
                'network_security_policy_ref' : network_security_ref
            }
        else:
            vs_obj = {
                'name' : vs['name'],
                'type' : 'VS_TYPE_NORMAL',
                'ip_address' : {
                    'addr' : vs['vip'],
                    'type' : 'V4'
                },
                'enabled' : True,
                'services' : service_objs,
                'pool_ref' : pool_ref,
                'application_profile_ref' : app_profile_ref
            }
        session.create_or_update(obj='virtualservice', 
                                 obj_name=vs['name'],
                                 data=vs_obj)

def create_avi_endpoint(tenant):
    controller_config = get_avi_controller_config()
    print ('Connecting to Avi Controller %s...'%(controller_config['ip']))
    print ('User : %s Tenant : %s' %(controller_config['username'],tenant))
    return ApiSession.get_session(controller_config['ip'],
                       controller_config['username'], 
                       controller_config['password'], tenant=tenant)


def create_application_profile(sess, name, pki_profile_obj_name):
    '''
    Create a L7 application profile with caching, compression and
    websockets enabled.
    '''
    pki_profile_ref = sess.get_object_uri('pkiprofile', pki_profile_obj_name)
    application_profile_obj = {
        'name' : name,
        'type' : 'APPLICATION_PROFILE_TYPE_HTTP',
        'http_profile' : {
            'connection_multiplexing_enabled' : True,
            'compression_profile' : {
                'compression' : True,
                'remove_accept_encoding_header' : False,
                'type' : 'AUTO_COMPRESSION'
            },
            'cache_config': {
                'enabled' : True,
            },
            'websockets_enabled' : True,
            'ssl_client_certificate_mode': 'SSL_CLIENT_CERTIFICATE_REQUEST',
            'ssl_client_certificate_action' : {
                'headers': [
                     {
                         'request_header' : 'X-Cert',
                         'request_header_value' : 'HTTP_POLICY_VAR_SSL_CLIENT_SUBJECT'
                     }
                ],
                'close_connection' : True
            },
            'pki_profile_ref' : pki_profile_ref
        }
    }
    rsp = sess.post('applicationprofile', 
                    data=json.dumps(application_profile_obj))
    return rsp


def create_pki_profile(sess, name, certs=[], crls=[]):
    cert_objs = []
    for cert in certs:
        cert_objs.append({
            'certificate' : cert
        })
    crl_objs = []
    for crl in crls:
        crl_objs.append({
            'body' : crl
        })
    pki_profile_obj = {
        'name' : name,
        'ca_certs' : cert_objs,
        'crls' : crl_objs
    }
    rsp = sess.post('pkiprofile', json.dumps(pki_profile_obj))
    return rsp

if __name__ == '__main__':

    with open('../certs/server.crt') as f:
        server_crt = f.read()
    with open('../certs/server.key') as f:
        server_key = f.read()
    with open('../certs/cakey.pem') as f:
        ca_key = f.read()
    with open('../certs/cacert.pem') as f:
        ca_cert = f.read()

    # Read the tenant config from apic/apic_tenant_config.json
    load_tenant_config()

    # Establish session with the Avi Controller in 'admin' tenant for
    # configuration
    try :
        admin_session = create_avi_endpoint(tenant='admin')
    except Exception: 
        print ('login failed to Avi Controller!')
        sys.exit(0) 

    ApiUtils(admin_session).import_ssl_certificate('MyCert', 
                                                   server_key, server_crt)
    create_pki_profile(admin_session, 'MyPKIProfile', certs=[ ca_cert ])
    create_application_profile(admin_session, 'MyAppProfile', 'MyPKIProfile')

    # Establish session with the APIC
    apic_config = get_avi_apic_config()
    apic_session = APICClient(
                     apic_config['ip'],
                     apic_config['username'],
                     apic_config['password'])
    create_apic_topology()

    print ('------------------------------------------------------')
    print ('Creating objects in Avi for Tenant %s' %(tenant_config['Tenant']))
    print ('------------------------------------------------------')
    avi_tenant_session = create_avi_endpoint(tenant=tenant_config['Tenant'])
    # Create Ip Address Group if present
    create_avi_ipaddrgroup(avi_tenant_session)

    # Vs/Pool are created in the context of the Tenant
    create_avi_pool(avi_tenant_session)
    create_avi_virtualservice(avi_tenant_session, 'MyAppProfile')

    print ('Finished creating Tenant %s' %(tenant_config['Tenant']))



import os
import sys
import copy
import json
import logging
from datetime import datetime, timedelta
from requests import ConnectionError
from requests import Response
from requests.sessions import Session

logger = logging.getLogger(__name__)


def avi_timedelta(td):
    '''
    This is a wrapper class to workaround python 2.6 builtin datetime.timedelta
    does not have total_seconds method
    :param timedelta object
    '''
    if type(td) != timedelta:
        raise TypeError()
    if sys.version_info >= (2, 7):
        ts = td.total_seconds()
    else:
        ts = td.seconds + (24 * 3600 * td.days)
    return ts


def avi_sdk_syslog_logger(logger_name='avi.sdk'):
    # The following sets up syslog module to log underlying avi SDK messages
    # based on the environment variables:
    #   AVI_LOG_HANDLER: names the logging handler to use. Only syslog is
    #     supported.
    #   AVI_LOG_LEVEL: Logging level used for the avi SDK. Default is DEBUG
    #   AVI_SYSLOG_ADDRESS: Destination address for the syslog handler.
    #   Default is /dev/log
    from logging.handlers import SysLogHandler
    lf = '[%(asctime)s] %(levelname)s [' \
                        '%(module)s.%(funcName)s:%(lineno)d] %(message)s'
    log = logging.getLogger(logger_name)
    log_level = os.environ.get('AVI_LOG_LEVEL', 'DEBUG')
    if log_level:
        log.setLevel(getattr(logging, log_level))
    formatter = logging.Formatter(lf)
    sh = SysLogHandler(address=os.environ.get('AVI_SYSLOG_ADDRESS', '/dev/log'))
    sh.setFormatter(formatter)
    log.addHandler(sh)
    return log


class ObjectNotFound(Exception):
    pass


class APIError(Exception):
    def __init__(self, arg, rsp=None):
        self.args = [arg, rsp]
        self.rsp = rsp


class AviServerError(APIError):
    def __init__(self, arg, rsp=None):
        super(AviServerError, self).__init__(arg, rsp)


class APINotImplemented(Exception):
    pass


class ApiResponse(Response):
    """
    Returns copy of the requests.Response object provides additional helper
    routines
        1. obj: returns dictionary of Avi Object
    """
    def __init__(self, rsp):
        super(ApiResponse, self).__init__()
        for k, v in rsp.__dict__.iteritems():
            setattr(self, k, v)

    def json(self):
        """
        Extends the session default json interface to handle special errors
        and raise Exceptions
        returns the Avi object as a dictionary from rsp.text
        """
        if self.status_code in (200, 201):
            if not self.text:
                # In cases like status_code == 201 the response text could be
                # empty string.
                return None
            return super(ApiResponse, self).json()
        elif self.status_code == 204:
            # No response needed; e.g., delete operation
            return None
        elif self.status_code == 404:
            raise ObjectNotFound()
        elif self.status_code >= 500:
            raise AviServerError('HTTP Error: %d Error Msg %s' % (
                                    self.status_code, self.text), self)
        else:
            raise APIError('HTTP Error: %d Error Msg %s' % (
                    self.status_code, self.text), self)

    def count(self):
        """
        return the number of objects in the collection response. If it is not
        a collection response then it would simply return 1.
        """
        obj = self.json()
        if 'count' in obj:
            # this was a resposne to collection
            return obj['count']
        return 1

    @staticmethod
    def to_avi_response(resp):
        if type(resp) == Response:
            return ApiResponse(resp)
        return resp


class ApiSession(Session):
    """
    Extends the Request library's session object to provide helper
    utilities to work with Avi Controller like authentication, api massaging
    etc.
    """
    sessionDict = {}
    # This keeps track of the process which created the cache.
    # At anytime the pid of the process changes then it would create
    # a new cache for that process.
    AVI_SLUG = 'Slug'
    SESSION_CACHE_EXPIRY = 20*60
    SHARED_USER_HDRS = ['X-CSRFToken', 'Session-Id']
    MAX_API_RETRIES = 3

    def __init__(self, controller_ip, username, password=None, token=None,
                 tenant=None, tenant_uuid=None, verify=False, port=None,
                 timeout=60, api_version=None,
                 retry_conxn_errors=False):
        """
        initialize new session object with authenticated token from login api.
        It also keeps a cache of user sessions that are cleaned up if inactive
        for more than 20 mins.

        Notes:
        01. If mode is https and port is none or 443, we don't embed the
            port in the prefix. The prefix would be 'https://ip'. If port
            is a non-default value then we concatenate https://ip:port
            in the prefix.
        02. If mode is http and the port is none or 80, we don't embed the
            port in the prefix. The prefix would be 'http://ip'. If port is
            a non-default value, then we concatenate http://ip:port in
            the prefix.
        """
        super(ApiSession, self).__init__()
        self.controller_ip = controller_ip
        self.username = username
        self.password = password
        self.keystone_token = token
        self.tenant_uuid = tenant_uuid
        self.tenant = tenant if tenant else "admin"
        self.headers = {}
        self.verify = verify
        self.port = port
        self.key = controller_ip + ":" + username
        self.api_version = api_version
        self.retry_conxn_errors = retry_conxn_errors

        # Refer Notes 01 and 02
        if controller_ip.startswith('http'):
            if port is None or port == 80:
                self.prefix = controller_ip
            else:
                self.prefix = '{x}:{y}'.format(x=controller_ip, y=port)
        else:
            if port is None or port == 443:
                self.prefix = 'https://{x}'.format(x=controller_ip)
            else:
                self.prefix = 'https://{x}:{y}'.format(x=controller_ip, y=port)
        self.timeout = timeout
        try:
            user_session = ApiSession.sessionDict[self.key]["api"]
        except KeyError:
            logger.debug("Session does not exist; Creating new session for %s",
                         self.key)
            self.authenticate_session()
            ApiSession.sessionDict[self.key] = \
                {"api": self, "last_used": datetime.utcnow()}
            user_session = self
        # don't save the tenant headers as it would interfere with the
        # individual method tenant overrides
        for hdr in self.SHARED_USER_HDRS:
            if hdr in user_session.headers:
                self.headers[hdr] = user_session.headers[hdr]
        self.cookies = user_session.cookies
        self.num_session_retries = 0
        self.pid = os.getpid()
        ApiSession._clean_inactive_sessions()
        return

    @staticmethod
    def get_session(controller_ip, username, password=None, token=None,
                    tenant=None, tenant_uuid=None, verify=False, port=None,
                    timeout=60, retry_conxn_errors=False):
        """
        returns the session object for same user and tenant
        calls init if session dose not exist and adds it to session cache
        :param controller_ip: controller IP address
        :param username:
        :param password:
        :param token: Token to use; example, a valid keystone token
        :param tenant: Name of the tenant on Avi Controller
        :param tenant_uuid: Don't specify tenant when using tenant_id
        :param port: Rest-API may use a different port other than 443
        :param timeout: timeout for API calls; Default value is 60 seconds
        """
        key = controller_ip + ":" + username
        try:
            user_session = ApiSession.sessionDict[key]["api"]
            tenant = tenant if tenant else 'admin'
            if (user_session.password != password or
                    user_session.keystone_token != token or
                    user_session.tenant != tenant or
                    user_session.tenant_uuid != tenant_uuid):
                logger.debug('Api Session auth credential mismatch %s', key)
                del ApiSession.sessionDict[key]
                user_session = None
        except KeyError:
            logger.debug("Session does not exist; Creating new session for %s",
                         key)
            user_session = None

        if not user_session:
            user_session = ApiSession(
                controller_ip, username, password, token=token, tenant=tenant,
                tenant_uuid=tenant_uuid, verify=verify, port=port,
                timeout=timeout,
                retry_conxn_errors=retry_conxn_errors)
            ApiSession.sessionDict[key] = \
                {"api": user_session, "last_used": datetime.utcnow()}
        ApiSession._clean_inactive_sessions()
        return user_session

    def reset_session(self):
        """
        resets and re-authenticates the current session.
        :param api: ApiSession object
        """
        logger.info('resetting session for %s', self.key)
        self.headers = {}
        self.authenticate_session()

    def authenticate_session(self):
        """
        Performs session authentication with Avi controller and stores
        session cookies and sets header options like tenant.
        """
        body = {"username": self.username}
        if not self.keystone_token:
            body["password"] = self.password
        else:
            body["token"] = self.keystone_token

        logger.debug('authenticating user %s ', self.username)
        rsp = super(ApiSession, self).post(self.prefix+"/login", body,
                                           timeout=self.timeout)
        if rsp.status_code != 200:
            raise Exception(
                "Authentication failed with code %d reason msg: %s" %
                (rsp.status_code, rsp.text))
        logger.debug("rsp cookies: %s", dict(rsp.cookies))
        self.headers.update({
            "Referer": self.prefix,
            "Content-Type": "application/json"
        })
        if rsp.cookies and 'csrftoken' in rsp.cookies:
            csrftoken = rsp.cookies['csrftoken']
            self.headers.update({"X-CSRFToken": csrftoken})
            if self.key in ApiSession.sessionDict:
                cached_api = \
                    ApiSession.sessionDict[self.key]['api']
                cached_api.headers.update({"X-CSRFToken": csrftoken})
        logger.debug("authentication success for user %s with headers: %s",
                     self.username, self.headers)
        return

    def _get_api_headers(self, tenant, tenant_uuid, timeout, headers, api_version):
        """
        returns the headers that are passed to the requests.Session api calls.
        """
        api_hdrs = copy.deepcopy(self.headers)
        api_hdrs['timeout'] = str(timeout)
        if api_version:
            api_hdrs['X-Avi-Version'] = api_version
        elif self.api_version:
            api_hdrs['X-Avi-Version'] = self.api_version
        if tenant:
            tenant_uuid = None
        elif tenant_uuid:
            tenant = None
        else:
            tenant = self.tenant
            tenant_uuid = self.tenant_uuid
        if tenant_uuid:
            api_hdrs.update({"X-Avi-Tenant-UUID": "%s" % tenant_uuid})
            api_hdrs.pop("X-Avi-Tenant", None)
        elif tenant:
            api_hdrs.update({"X-Avi-Tenant": "%s" % tenant})
            api_hdrs.pop("X-Avi-Tenant-UUID", None)
        if headers:
            # overwrite the headers passed via the API calls.
            api_hdrs.update(headers)
        return api_hdrs

    def _api(self, api_name, path, tenant, tenant_uuid, data=None,
             headers=None, timeout=None, api_version=None, **kwargs):
        """
        It calls the requests.Session APIs and handles session expiry
        and other situations where session needs to be reset.
        returns ApiResponse object
        :param path: takes relative path to the AVI api.
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param headers: dictionary of headers that override the session
            headers.
        """
        if self.pid != os.getpid():
            logger.info('pid %d change detected new %d. Closing session',
                        self.pid, os.getpid())
            self.close()
            self.pid = os.getpid()
        if timeout is None:
            timeout = self.timeout
        fullpath = self._get_api_path(path)
        fn = getattr(super(ApiSession, self), api_name)
        api_hdrs = \
            self._get_api_headers(tenant, tenant_uuid, timeout, headers, api_version)
        connection_error = False
        try:
            if (data is not None) and (type(data) == dict):
                resp = fn(fullpath, data=json.dumps(data), headers=api_hdrs,
                          timeout=timeout, **kwargs)
            else:
                resp = fn(fullpath, data=data, headers=api_hdrs,
                          timeout=timeout, **kwargs)
        except ConnectionError as e:
            logger.warning('Connection error retrying %s', e)
            if not self.retry_conxn_errors:
                raise
            connection_error = True
        except Exception as e:
            logger.error('Error in Requests library %s', e)
            raise
        if not connection_error:
            logger.debug(
                'path: %s http_method: %s hdrs: %s params: %s data: %s rsp: %s',
                fullpath, api_name.upper(), api_hdrs, kwargs, data, resp.text)
        if connection_error or resp.status_code in (401, 419):
            if connection_error:
                logger.warning('Connection failed, retrying.')
            else:
                logger.info('received error %d %s so resetting connection',
                            resp.status_code, resp.text)
            ApiSession.reset_session(self)
            self.num_session_retries += 1
            if self.num_session_retries > self.MAX_API_RETRIES:
                # Added this such that any code which re-tries can succeed
                # eventually.
                self.num_session_retries = 0
                raise APIError(
                    "giving up after %d retries connection failure %s" %
                    (self.MAX_API_RETRIES, connection_error))
            # should restore the updated_hdrs to one passed down
            resp = self._api(api_name, path, tenant, tenant_uuid, data,
                             headers=headers, api_version=api_version,
                             timeout=timeout, **kwargs)
            self.num_session_retries = 0

        if resp.cookies and 'csrftoken' in resp.cookies:
            csrftoken = resp.cookies['csrftoken']
            self.headers.update({"X-CSRFToken": csrftoken})
        self._update_session_last_used()
        return ApiResponse.to_avi_response(resp)

    def get(self, path, tenant='', tenant_uuid='', timeout=None, params=None,
            api_version=None, **kwargs):
        """
        It extends the Session Library interface to add AVI API prefixes,
        handle session exceptions related to authentication and update
        the global user session cache.
        :param path: takes relative path to the AVI api.
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        get method takes relative path to service and kwargs as per Session
            class get method
        returns session's response object
        """
        return self._api('get', path, tenant, tenant_uuid, timeout=timeout,
                         params=params, api_version=api_version, **kwargs)

    def get_object_by_name(self, path, name, tenant='', tenant_uuid='',
                           timeout=None, params=None, api_version=None, **kwargs):
        """
        Helper function to access Avi REST Objects using object
        type and name. It behaves like python dictionary interface where it
        returns None when the object is not present in the AviController.
        Internally, it transforms the request to api/path?name=<name>...
        :param path: relative path to service
        :param name: name of the object
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns dictionary object if successful else None
        """
        obj = None
        if not params:
            params = {}
        params['name'] = name
        resp = self.get(path, tenant, tenant_uuid, timeout=timeout,
                        params=params, api_version=api_version, **kwargs)
        if resp.status_code in (401, 419):
            ApiSession.reset_session(self)
            resp = self.get_object_by_name(
                    path, name, tenant, tenant_uuid, timeout=timeout,
                    params=params, **kwargs)
        if resp.status_code > 299:
            return obj
        try:
            obj = resp.json()['results'][0]
        except IndexError:
            obj = None
        self._update_session_last_used()
        return obj

    def post(self, path, data=None, tenant='', tenant_uuid='', timeout=None,
             force_uuid=None, params=None, api_version=None, **kwargs):
        """
        It extends the Session Library interface to add AVI API prefixes,
        handle session exceptions related to authentication and update
        the global user session cache.
        :param path: takes relative path to the AVI api.It is modified by
        the library to conform to AVI Controller's REST API interface
        :param data: dictionary of the data. Support for json string
            is deprecated
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns session's response object
        """
        if force_uuid is not None:
            headers = kwargs.get('headers', {})
            headers[self.AVI_SLUG] = force_uuid
            kwargs['headers'] = headers
        return self._api('post', path, tenant, tenant_uuid, data=data,
                         timeout=timeout, params=params, api_version=api_version, **kwargs)

    def put(self, path, data=None, tenant='', tenant_uuid='',
            timeout=None, params=None, api_version=None, **kwargs):
        """
        It extends the Session Library interface to add AVI API prefixes,
        handle session exceptions related to authentication and update
        the global user session cache.
        :param path: takes relative path to the AVI api.It is modified by
            the library to conform to AVI Controller's REST API interface
        :param data: dictionary of the data. Support for json string
            is deprecated
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns session's response object
        """
        return self._api('put', path, tenant, tenant_uuid, data=data,
                         timeout=timeout, params=params, api_version=api_version, **kwargs)

    def patch(self, path, data=None, tenant='', tenant_uuid='',
              timeout=None, params=None, api_version=None, **kwargs):
        """
        It extends the Session Library interface to add AVI API prefixes,
        handle session exceptions related to authentication and update
        the global user session cache.
        :param path: takes relative path to the AVI api.It is modified by
            the library to conform to AVI Controller's REST API interface
        :param data: dictionary of the data. Support for json string
            is deprecated
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns session's response object
        """
        return self._api('patch', path, tenant, tenant_uuid, data=data,
                         timeout=timeout, params=params, api_version=api_version, **kwargs)

    def put_by_name(self, path, name, data=None, tenant='',
                    tenant_uuid='', timeout=None, params=None, api_version=None, **kwargs):
        """
        Helper function to perform HTTP PUT on Avi REST Objects using object
        type and name.
        Internally, it transforms the request to api/path?name=<name>...
        :param path: relative path to service
        :param name: name of the object
        :param data: dictionary of the data. Support for json string
            is deprecated
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns session's response object
        """
        uuid = self._get_uuid_by_name(path, name, tenant, tenant_uuid)
        path = '%s/%s' % (path, uuid)
        return self.put(path, data, tenant, tenant_uuid, timeout=timeout,
                        params=params, api_version=api_version, **kwargs)

    def delete(self, path, tenant='', tenant_uuid='', timeout=None, params=None,
               data=None, api_version=None, **kwargs):
        """
        It extends the Session Library interface to add AVI API prefixes,
        handle session exceptions related to authentication and update
        the global user session cache.
        :param path: takes relative path to the AVI api.It is modified by
        the library to conform to AVI Controller's REST API interface
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param data: dictionary of the data. Support for json string
            is deprecated
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns session's response object
        """
        return self._api('delete', path, tenant, tenant_uuid, data=data,
                         timeout=timeout, params=params, api_version=api_version, **kwargs)

    def delete_by_name(self, path, name, tenant='', tenant_uuid='', timeout=None,
                       params=None, api_version=None, **kwargs):
        """
        Helper function to perform HTTP DELETE on Avi REST Objects using object
        type and name.Internally, it transforms the request to
        api/path?name=<name>...
        :param path: relative path to service
        :param name: name of the object
        :param tenant: overrides the tenant used during session creation
        :param tenant_uuid: overrides the tenant or tenant_uuid during session
            creation
        :param timeout: timeout for API calls; Default value is 60 seconds
        :param params: dictionary of key value pairs to be sent as query
            parameters
        :param api_version: overrides x-avi-header in request header during
            session creation
        returns session's response object
        """
        uuid = self._get_uuid_by_name(path, name, tenant, tenant_uuid)
        if not uuid:
            raise ObjectNotFound("%s/?name=%s" % (path, name))
        path = '%s/%s' % (path, uuid)
        return self.delete(path, tenant, tenant_uuid, timeout=timeout,
                           params=params, api_version=api_version, **kwargs)

    def get_obj_ref(self, obj):
        """returns reference url from dict object"""
        if not obj:
            return None
        if isinstance(obj, Response):
            obj = json.loads(obj.text)
        if obj.get(0, None):
            return obj[0]['url']
        elif obj.get('url', None):
            return obj['url']
        elif obj.get('results', None):
            return obj['results'][0]['url']
        else:
            return None

    def get_obj_uuid(self, obj):
        """returns uuid from dict object"""
        if not obj:
            raise ObjectNotFound()
        if isinstance(obj, Response):
            obj = json.loads(obj.text)
        if obj.get(0, None):
            return obj[0]['uuid']
        elif obj.get('uuid', None):
            return obj['uuid']
        elif obj.get('results', None):
            return obj['results'][0]['uuid']
        else:
            return None

    def _get_api_path(self, path, uuid=None):
        """
        This function returns the full url from relative path and uuid.
        """
        if uuid:
            return self.prefix+'/api/'+path+'/'+uuid
        else:
            return self.prefix+'/api/'+path

    def _get_uuid_by_name(self, path, name, tenant='admin', tenant_uuid=''):
        """gets object by name and service path and returns uuid"""
        resp = self.get_object_by_name(path, name, tenant, tenant_uuid)
        if not resp:
            raise ObjectNotFound("%s/%s" % (path, name))
        return self.get_obj_uuid(resp)

    def _update_session_last_used(self):
        if self.key in ApiSession.sessionDict:
            ApiSession.sessionDict[self.key]["last_used"] = \
                datetime.utcnow()
        else:
            ApiSession.sessionDict[self.key] = \
                {'api': self, 'last_used': datetime.utcnow()}

    @staticmethod
    def _clean_inactive_sessions():
        """Removes sessions which are inactive more than 20 min"""
        session_cache = ApiSession.sessionDict
        logger.debug("cleaning inactive sessions in pid %d num elem %d",
                     os.getpid(), len(session_cache))
        keys_to_delete = []
        for key, session in session_cache.iteritems():
            tdiff = avi_timedelta(session["last_used"] - datetime.utcnow())
            if tdiff < ApiSession.SESSION_CACHE_EXPIRY:
                continue
            keys_to_delete.append(key)
        for key in keys_to_delete:
            del session_cache[key]
            logger.debug("Removed session for : %s", key)

    def delete_session(self):
        """ Removes the session for cleanup"""
        logger.debug("Removed session for : %s", self.key)
        ApiSession.sessionDict.pop(self.key, None)
        return
# End of file



import json
import os
import requests
import sys
import argparse
import time

from avi.sdk.avi_api import ApiSession

def backup_configuration(api, args):
    data = {'passphrase': args.passphrase} if args.passphrase else {}
    params = {'full_system': True}
    filename = "%s_%s.%s" % (args.filename, args.controller, time.strftime("%Y%m%d_%H%M%S"))
    rsp = api.post('configuration/export', params=params, data=json.dumps(data))
    with open(filename, 'w') as f:
        f.write(rsp.text)
    print 'Configuration successfully saved in %s' % filename

def main(args):
    api = ApiSession.get_session(args.controller, args.username, args.password)
    backup_configuration(api, args)

if __name__ == "__main__":
    requests.packages.urllib3.disable_warnings()
    parser = argparse.ArgumentParser(description="Script to get the full configuration from Avi controller")
    parser.add_argument("-u", "--username", required=True,
                        help="Login username")
    parser.add_argument("-p", "--password", required=True,
                        help="Login password")
    parser.add_argument("-c", "--controller", required=True,
                        help="Controller IP address")
    parser.add_argument("-P", "--passphrase", required=False,
                        help="Passphrase to encrypt sensitive information in backup")
    parser.add_argument("-f", "--filename", required=True,
                        help="Filename prefix - will be suffixed with a timestamp")
    args = parser.parse_args()
    main(args)
